searchNodes=[{"doc":"Bolerplate to effectively handle many long-lived entities of the same shape, driven by FSM. Siblings is a library to painlessly manage many uniform processes, all having the lifecycle and the FSM behind. Consider the service, that polls the market rates from several diffferent sources, allowing semi-automated trading based on predefined conditions. For each bid, the process is to be spawn, polling the external resources. Once the bid condition is met, the bid gets traded. With Siblings , one should implement Siblings.Worker.perform/3 callback, doing actual work and returning either :ok if no action should be taken, or {:transition, event, payload} to initiate the FSM transition. When the FSM get exhausted (reaches its end state,) both the performing process and the FSM itself do shut down. FSM instances leverage Finitomata library, which should be used alone if no recurrent perform should be accomplished or if the instances are not uniform. Typical code for the Siblings.Worker implementation would be as follows defmodule MyApp.Worker do @fsm &quot;&quot;&quot; born --&gt; |reject| rejected born --&gt; |bid| traded &quot;&quot;&quot; use Finitomata , @fsm def on_transition ( :born , :reject , _nil , payload ) do perform_rejection ( payload ) { :ok , :rejected , payload } end def on_transition ( :born , :bid , _nil , payload ) do perform_bidding ( payload ) { :ok , :traded , payload } end @behaviour Siblings.Worker @impl Siblings.Worker def perform ( state , id , payload ) def perform ( :born , id , payload ) do cond do time_to_bid? ( ) -&gt; { :transition , :bid , nil } stale? ( ) -&gt; { :transition , :reject , nil } true -&gt; :ok end end def perform ( :rejected , id , _payload ) do Logger . info ( &quot;The bid \#{ id } was rejected&quot; ) { :transition , :__end__ , nil } end def perform ( :traded , id , _payload ) do Logger . info ( &quot;The bid \#{ id } was traded&quot; ) { :transition , :__end__ , nil } end end Now it can be used as shown below { :ok , pid } = Siblings . start_link ( ) Siblings . start_child ( MyApp.Worker , &quot;Bid1&quot; , %{ } , interval : 1_000 ) Siblings . start_child ( MyApp.Worker , &quot;Bid2&quot; , %{ } , interval : 1_000 ) ... The above would spawn two processes, checking the conditions once per a second ( interval ,) and manipulating the underlying FSM to walk through the bids’ lifecycles. Worker’s interval might be reset with GenServer.cast(pid, {:reset, interval}) and the message might be casted to it with GenServer.call(pid, {:message, message}) . For the latter to work, the optional callback on_call/2 must be implemented. Sidenote: Normally, Siblings supervisor would be put into the supervision tree of the target application.","ref":"Siblings.html","title":"Siblings","type":"module"},{"doc":"Performs a GenServer.call/3 on the named worker.","ref":"Siblings.html#call/3","title":"Siblings.call/3","type":"function"},{"doc":"Returns the child spec for the named or unnamed Siblings process. Useful when many Siblings processes are running simultaneously.","ref":"Siblings.html#child_spec/1","title":"Siblings.child_spec/1","type":"function"},{"doc":"Returns the list of currently managed children.","ref":"Siblings.html#children/2","title":"Siblings.children/2","type":"function"},{"doc":"Returns the child with the given id , or nil if there is none.","ref":"Siblings.html#find_child/3","title":"Siblings.find_child/3","type":"function"},{"doc":"Performs a GenServer.call/3 on all the workers.","ref":"Siblings.html#multi_call/2","title":"Siblings.multi_call/2","type":"function"},{"doc":"Initiates the transition of all the workers.","ref":"Siblings.html#multi_transition/3","title":"Siblings.multi_transition/3","type":"function"},{"doc":"Returns the payload of FSM behind the named worker.","ref":"Siblings.html#payload/2","title":"Siblings.payload/2","type":"function"},{"doc":"Resets the the named worker’s interval.","ref":"Siblings.html#reset/3","title":"Siblings.reset/3","type":"function"},{"doc":"Starts the supervised child under the PartitionSupervisor .","ref":"Siblings.html#start_child/4","title":"Siblings.start_child/4","type":"function"},{"doc":"Starts the supervision subtree, holding the PartitionSupervisor .","ref":"Siblings.html#start_link/1","title":"Siblings.start_link/1","type":"function"},{"doc":"Returns the state of the named worker.","ref":"Siblings.html#state/2","title":"Siblings.state/2","type":"function"},{"doc":"Initiates the transition of the named worker.","ref":"Siblings.html#transition/4","title":"Siblings.transition/4","type":"function"},{"doc":"The worker for the single sibling process.","ref":"Siblings.Worker.html","title":"Siblings.Worker","type":"behaviour"},{"doc":"The Finitomata FSM implementation module. It will be used internally to carry the state of FSM. If not implemented, this module itself will be considered an FSM implementation.","ref":"Siblings.Worker.html#c:fsm/0","title":"Siblings.Worker.fsm/0","type":"callback"},{"doc":"The handler for the routed message from Siblings.InternalWorker.handle_call({:message, any()}) .","ref":"Siblings.Worker.html#c:on_call/2","title":"Siblings.Worker.on_call/2","type":"callback"},{"doc":"The function to re-initialize FSM after crash.","ref":"Siblings.Worker.html#c:on_init/1","title":"Siblings.Worker.on_init/1","type":"callback"},{"doc":"The callback to be implemented in each and every worker. It will be called internally continiously by the internal worker process, transitioning the underlying FSM according to the return value.","ref":"Siblings.Worker.html#c:perform/3","title":"Siblings.Worker.perform/3","type":"callback"},{"doc":"Value, returned from on_call/2 callback","ref":"Siblings.Worker.html#t:call_result/0","title":"Siblings.Worker.call_result/0","type":"type"},{"doc":"Identifier of the worker process","ref":"Siblings.Worker.html#t:id/0","title":"Siblings.Worker.id/0","type":"type"},{"doc":"Message to be sent to the worker process","ref":"Siblings.Worker.html#t:message/0","title":"Siblings.Worker.message/0","type":"type"},{"doc":"Payload associated with the worker","ref":"Siblings.Worker.html#t:payload/0","title":"Siblings.Worker.payload/0","type":"type"},{"doc":"The internal process to manage Siblings.Worker subsequent runs along with its FSM .","ref":"Siblings.InternalWorker.html","title":"Siblings.InternalWorker","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Siblings.InternalWorker.html#child_spec/1","title":"Siblings.InternalWorker.child_spec/1","type":"function"},{"doc":"Allowed options in a call to start_link/4","ref":"Siblings.InternalWorker.html#t:options/0","title":"Siblings.InternalWorker.options/0","type":"type"},{"doc":"The state of the worker.","ref":"Siblings.InternalWorker.State.html","title":"Siblings.InternalWorker.State","type":"module"},{"doc":"","ref":"Siblings.InternalWorker.State.html#t:t/0","title":"Siblings.InternalWorker.State.t/0","type":"type"},{"doc":"The partitioned dynamic supervision of FSM-backed workers.","ref":"readme.html","title":"Siblings","type":"extras"},{"doc":"Siblings is a library to painlessly manage many uniform processes, all having the lifecycle and the FSM behind. Consider the service, that polls the market rates from several diffferent sources, allowing semi-automated trading based on predefined conditions. For each bid, the process is to be spawn, polling the external resources. Once the bid condition is met, the bid gets traded. With Siblings , one should implement Siblings.Worker.perform/3 callback, doing actual work and returning either :ok if no action should be taken, or {:transition, event, payload} to initiate the FSM transition. When the FSM get exhausted (reaches its end state,) both the performing process and the FSM itself do shut down. FSM instances leverage Finitomata library, which should be used alone if no recurrent perform should be accomplished or if the instances are not uniform. Typical code for the Siblings.Worker implementation would be as follows defmodule MyApp.Worker do @fsm &quot;&quot;&quot; born --&gt; |reject| rejected born --&gt; |bid| traded &quot;&quot;&quot; use Finitomata , @fsm def on_transition ( :born , :reject , _nil , payload ) do perform_rejection ( payload ) { :ok , :rejected , payload } end def on_transition ( :born , :bid , _nil , payload ) do perform_bidding ( payload ) { :ok , :traded , payload } end @behaviour Siblings.Worker @impl Siblings.Worker def perform ( state , id , payload ) def perform ( :born , id , payload ) do cond do time_to_bid? ( ) -&gt; { :transition , :bid , nil } stale? ( ) -&gt; { :transition , :reject , nil } true -&gt; :ok end end def perform ( :rejected , id , _payload ) do Logger . info ( &quot;The bid \#{ id } was rejected&quot; ) { :transition , :__end__ , nil } end def perform ( :traded , id , _payload ) do Logger . info ( &quot;The bid \#{ id } was traded&quot; ) { :transition , :__end__ , nil } end end Now it can be used as shown below { :ok , pid } = Siblings . start_link ( ) Siblings . start_child ( MyApp.Worker , &quot;Bid1&quot; , %{ } , interval : 1_000 ) Siblings . start_child ( MyApp.Worker , &quot;Bid2&quot; , %{ } , interval : 1_000 ) ... The above would spawn two processes, checking the conditions once per a second ( interval ,) and manipulating the underlying FSM to walk through the bids’ lifecycles. Worker’s interval might be reset with GenServer.cast(pid, {:reset, interval}) and the message might be casted to it with GenServer.call(pid, {:message, message}) . For the latter to work, the optional callback on_call/2 must be implemented. Sidenote: Normally, Siblings supervisor would be put into the supervision tree of the target application.","ref":"readme.html#usage","title":"Siblings - Usage","type":"extras"},{"doc":"def deps do [ { :siblings , &quot;~&gt; 0.1&quot; } ] end","ref":"readme.html#installation","title":"Siblings - Installation","type":"extras"},{"doc":"0.4.2 — accept workers: in call to Siblings.child_spec/1 to statically initialize Siblings 0.4.1 — [BUG] many named Siblings instances 0.4.0 — Siblings.{multi_call/2, multi_transition/3} 0.3.3 — Siblings.{state/1, payload/2} 0.3.2 — Siblings.{call/3, reset/3, transition/4} 0.3.1 — retrieve childrens as both map and list 0.3.0 — GenServer.cast(pid, {:reset, interval}) and GenServer.call(pid, {:message, message}) 0.2.0  — Fast Worker lookup 0.1.0  — Initial MVP","ref":"readme.html#changelog","title":"Siblings - Changelog","type":"extras"},{"doc":"","ref":"readme.html#documentation","title":"Siblings - Documentation","type":"extras"}]